@@ -17,7 +17,7 @@ import os
 from peyecoder.models import Occluders, Subject
 from peyecoder.panels import LogTable
 from peyecoder.file_utils import load_datafile
-from peyecoder.export import export
+from peyecoder.export import export, INVERT_RESPONSE, INVERT_TRIAL_ORDER
 
 
 def get_save_filename(parent, caption, filter, default_suffix=''):
@@ -211,7 +211,14 @@ class SubjectDialog(QDialog):
         }
 
         self.parent().subject.update_from_dict(d)
+
+        # When the subject info dialog is first opened, sync_fields is called even if nothing has been entered by the
+        # user.  To avoid unnecessary prompts to save when nothing has been changed, only mark the record dirty if
+        # a datafile has been loaded.
+        if self.parent().filename:
+            self.parent().subject.dirty = True
         self.parent().update_info_panel()
+        self.parent().update_log()
 
     def update_from_dict(self, d):
         self.subject_box.setText(str(d.get('Number', '')))
@@ -275,17 +282,27 @@ class OccluderDialog(QDialog):
         self.table.setHorizontalHeaderLabels(('x', 'y', 'w', 'h'))
         self.table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
 
-        self.add_row_button = QPushButton('Add Occluder')
+        self.add_row_button = QPushButton('Add')
         self.add_row_button.clicked.connect(self.add_row)
 
-        self.save_occluders_button = QPushButton('Save Occluders')
+        self.save_occluders_button = QPushButton('Save')
         self.save_occluders_button.clicked.connect(self.save_occluders)
 
+        self.delete_row_button = QPushButton('Delete')
+        self.delete_row_button.clicked.connect(self.delete_current_row)
+
+        self.buttons = QHBoxLayout()
+        self.buttons.addWidget(self.delete_row_button)
+        self.buttons.addWidget(self.add_row_button)
+        self.buttons.addWidget(self.save_occluders_button)
+
+
         self.layout = QVBoxLayout()
         self.layout.addWidget(self.top_label)
         self.layout.addWidget(self.table)
-        self.layout.addWidget(self.add_row_button)
-        self.layout.addWidget(self.save_occluders_button)
+        self.layout.addLayout(self.buttons)
+        #self.layout.addWidget(self.add_row_button)
+        #self.layout.addWidget(self.save_occluders_button)
 
         self.setLayout(self.layout)
 
@@ -310,10 +327,18 @@ class OccluderDialog(QDialog):
                 # ValueError occurs for cells containing non-integer text
                 pass
         self.parent().subject.occluders = Occluders(occluders)
+        self.parent().subject.dirty = True
+        self.parent().vid.reload_buffer()
+        self.parent().show_frame()
 
     def add_row(self):
         self.table.setRowCount(self.table.rowCount() + 1)
 
+    def delete_current_row(self):
+        # require that _something_ is selected to indicate intent
+        if len(self.table.selectedRanges()) > 0:
+            self.table.removeRow(self.table.currentRow())
+
 
 class KeyTableWidget(QTableWidget):
     def __init__(self):
@@ -463,6 +488,9 @@ class SettingsDialog(QDialog):
         d['Toggle Trial Status Key'] = self.toggle_box.get_key()
         # Update step label on main form
         self.parent().step_label.setText('Step: {}'.format(d['Step']))
+        # update code responses to reflect current response keys
+        self.parent().code_tab.set_responses(list(d['Response Keys'].values()))
+        self.parent().subject.dirty = True
 
     def show(self):
         super().show()
@@ -553,9 +581,15 @@ class ExportDialog(QDialog):
         self.format_radiogroup = QButtonGroup()
         self.format_radiogroup.addButton(self.wide_radio, id=1)
         self.format_radiogroup.addButton(self.long_radio, id=2)
-
         self.wide_radio.setChecked(True)  # default wide
-        self.invert_checkbox = QCheckBox('Exchange R and L in trial order')
+
+        self.invert_label = QLabel('Left-Right Inversion:')
+        self.invert_trial_order_radio = QRadioButton('Invert Target Side and Images (iCoder)')
+        self.invert_response_radio = QRadioButton('Invert Responses')
+        self.invert_radiogroup = QButtonGroup()
+        self.invert_radiogroup.addButton(self.invert_trial_order_radio, id=INVERT_TRIAL_ORDER)
+        self.invert_radiogroup.addButton(self.invert_response_radio, id=INVERT_RESPONSE)
+        self.invert_trial_order_radio.setChecked(True)  # default to iCoder-style inversion
 
         self.button_box = QDialogButtonBox(QDialogButtonBox.Save | QDialogButtonBox.Cancel)
         self.button_box.accepted.connect(self.export_csv)
@@ -565,16 +599,17 @@ class ExportDialog(QDialog):
         layout.addWidget(self.format_label)
         layout.addWidget(self.wide_radio)
         layout.addWidget(self.long_radio)
-        layout.addWidget(self.invert_checkbox)
+        layout.addWidget(self.invert_label)
+        layout.addWidget(self.invert_trial_order_radio)
+        layout.addWidget(self.invert_response_radio)
         layout.addWidget(self.button_box)
-
         self.setLayout(layout)
 
     def export_csv(self):
         filename = get_save_filename(self, "Save CSV File", filter="CSV Files (*.csv)", default_suffix='csv')
         if filename != '':
             export_format = {1: 'wide', 2: 'long'}[self.format_radiogroup.checkedId()]
-            export(filename, self.parent().subject, format=export_format, invert_rl=self.invert_checkbox.isChecked())
+            export(filename, self.parent().subject, format=export_format, invert_rl=self.invert_radiogroup.checkedId())
             self.accept()
         else:
             self.reject()
