@@ -5,12 +5,13 @@ import math
 from PySide2 import QtCore, QtWidgets, QtGui
 from PySide2.QtWidgets import QLabel, QPushButton, QSlider, QStyle, \
     QHBoxLayout, QVBoxLayout, QSizePolicy, QAction, QGridLayout, QDialog, \
-    QTabWidget, QSplitter
+    QTabWidget, QSplitter, QScrollArea, QMessageBox
 from PySide2.QtGui import Qt
 from PySide2.QtCore import QObject, QEvent
 
 import timecode
 import os
+from functools import partial
 
 from peyecoder.video_reader import BufferedVideoReader
 from peyecoder.audio_player import VideoAudioPlayer
@@ -20,6 +21,7 @@ from peyecoder.file_utils import load_datafile, save_datafile
 from peyecoder.dialogs import SubjectDialog, TimecodeDialog, OccluderDialog, SettingsDialog, CodeComparisonDialog, \
     ReportDialog, ExportDialog, get_save_filename
 from peyecoder.reliability import reliability_report
+from peyecoder import version
 
 STATE_PLAYING = 1
 STATE_PAUSED = 2
@@ -59,7 +61,7 @@ class MainWindow(QtWidgets.QMainWindow):
         self.code_comparison_dialog = None
         self.report_dialog = None
 
-        self.subject = Subject()
+        self.subject = Subject(self)
 
         # Timecode object used to translate from frames to SMTP timecode
         # Initialize with default framerate; will be updated when a video is loaded
@@ -92,19 +94,30 @@ class MainWindow(QtWidgets.QMainWindow):
         splitter.setCollapsible(1, False)
         splitter.splitterMoved.connect(self.splitter_moved)
 
+        scrollable_message = QScrollArea()
+        scrollable_message.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
+        scrollable_message.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
+        scrollable_message.setWidgetResizable(True)
+        scrollable_message.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Maximum)
+        scrollable_message.setWidget(self.message_box)
+
         layout3 = QVBoxLayout()
         layout3.addWidget(splitter)
         layout3.addWidget(tab_widget)
         layout3.addLayout(info_grid)
-        layout3.addWidget(self.message_box)
 
-        # Create a widget for window contents
-        wid = QtWidgets.QWidget(self)
-        self.setCentralWidget(wid)
-        # wid.keyPressEvent = self.test_keypress_event
-        # wid.keyReleaseEvent = self.handle_keypress  # works for arrows, unlike keyPressEvent
+        container2 = QtWidgets.QWidget()
+        container2.setLayout(layout3)
 
-        wid.setLayout(layout3)
+        splitter2 = QSplitter()
+        splitter2.setOrientation(Qt.Vertical)
+        splitter2.addWidget(container2)
+        splitter2.addWidget(scrollable_message)
+        splitter2.setCollapsible(0, False)
+        splitter2.setCollapsible(1, False)
+        splitter2.splitterMoved.connect(self.splitter_moved)
+
+        self.setCentralWidget(splitter2)
 
         # Use event filter to cause mouse clicks away from the log table to
         # clear the selection (removes focus to fix key handling)
@@ -115,9 +128,33 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.reset_state()
 
+    def prompt_save(self):
+        proceed = True
+        if self.subject.dirty:
+            ret = QMessageBox.warning(self, 'Peyecoder', 'Do you want to save changes?',
+                                      QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel,
+                                      defaultButton=QMessageBox.Save)
+            if ret == QMessageBox.Save:
+                # save and, if successful, close
+                proceed = self.save_datafile()
+            else:
+                proceed = ret != QMessageBox.Cancel
+
+        return proceed
+
+    def closeEvent(self, event):
+        if self.prompt_save():
+            event.accept()
+        else:
+            event.ignore()
+
     def reset_state(self):
         """Initialize state or reset to initial state"""
-        self.subject = Subject()
+
+        if not self.prompt_save():
+            return
+
+        self.subject = Subject(self)
         self.filename = ''
         self.setWindowTitle('peyecoder')
         # reset video source
@@ -242,6 +279,7 @@ class MainWindow(QtWidgets.QMainWindow):
         self.logtable = LogTable()
         self.logtable.setColumnCount(3)
         self.logtable.setAlternatingRowColors(True)
+        self.logtable.setStyleSheet("QTableWidget::item:selected{background-color: palette(Highlight); color: palette(HighlightedText);};")
         self.logtable.setHorizontalHeaderLabels(self.logtable.Labels.Prescreen1)
 
         self.logtable.setMinimumWidth(400)
@@ -273,8 +311,10 @@ class MainWindow(QtWidgets.QMainWindow):
         self.active_tab = index
         self.update_log()
 
-    def update_log(self):
+    def update_log(self, preserve_highlight=False):
         """Update data and labels in log file"""
+        if preserve_highlight:
+            rows = self.logtable.selected_rows()
         if self.active_tab == TAB_PRESCREEN:
             self.logtable.set_prescreen_labels(self.prescreen_tab.prescreener())
             self.logtable.load_data(self.subject.reasons.render(self.prescreen_tab.prescreener()))
@@ -282,9 +322,15 @@ class MainWindow(QtWidgets.QMainWindow):
             self.logtable.set_code_labels()
             self.logtable.load_data(self.subject.events.render(self.subject.timecode_offsets, self.timecode))
 
-            errors, err_msg = self.subject.events.error_items(self.subject.trial_order.unused + self.subject.reasons.unused())
+            errors, err_msg = self.subject.events.error_items(self.subject.trial_order.unused + self.subject.reasons.unused(),
+                                                              self.subject.trial_order.max_trial)
             self.logtable.redden_rows(errors)
             self.message_box.setText('\n'.join(err_msg))
+        if preserve_highlight:
+            self.logtable.itemSelectionChanged.disconnect(self.select_code_row)
+            self.logtable.select_rows(rows)
+
+            self.logtable.itemSelectionChanged.connect(self.select_code_row)
 
     def select_code_row(self):
         """ When a row or rows have been selected in the code log, update the code tab widgets and the video position
@@ -308,14 +354,16 @@ class MainWindow(QtWidgets.QMainWindow):
         self.subject.reasons.add_reason(reason, ps=self.prescreen_tab.group_who.checkedId())
         self.update_log()
         self.update_info_panel()
+        self.subject.dirty = True
 
     def add_event(self, event):
         event.frame = self.vid.frame_number
         self.subject.events.add_event(event)
-        self.update_log()
+        self.update_log(preserve_highlight=True)
         # Scroll to the newly-added item
         row = self.subject.events.absolute_index(event)
         self.logtable.scroll_to_row(row)
+        self.subject.dirty = True
 
     def delete_data_rows(self, rows):
         """Delete rows from events or reasons as appropriate """
@@ -329,6 +377,7 @@ class MainWindow(QtWidgets.QMainWindow):
             # delete code entries by row (in descending order)
             for row in reversed(sorted(rows)):
                 self.subject.events.delete_event(row)
+        self.subject.dirty = True
 
     def build_menu(self):
         """Create the menu bar and global fixed actions"""
@@ -365,6 +414,37 @@ class MainWindow(QtWidgets.QMainWindow):
         reliability_action.setStatusTip('Compare coding against another file')
         reliability_action.triggered.connect(self.open_reliability_datafile)
 
+        prescreen_action = QAction('Pre-Screen', self)
+        prescreen_action.setShortcut('Ctrl+Shift+P')
+        prescreen_action.triggered.connect(partial(self.tab_widget.setCurrentIndex, TAB_PRESCREEN))
+
+        code_action = QAction('Code', self)
+        code_action.setShortcut('Ctrl+Shift+C')
+        code_action.triggered.connect(partial(self.tab_widget.setCurrentIndex, TAB_CODE))
+
+        next_step_action = QAction('Skip &forward', self)
+        next_step_action.setShortcut(']')
+        next_step_action.setShortcutContext(Qt.WidgetShortcut)  # prevent menu item from intercepting this keypress
+
+        prev_step_action = QAction('Skip &backward', self)
+        prev_step_action.setShortcut('[')
+        prev_step_action.setShortcutContext(Qt.WidgetShortcut)
+
+        increment_action = QAction('Increment Trial', self)
+        increment_action.setShortcut('+')
+        increment_action.setShortcutContext(Qt.WidgetShortcut)
+
+        decrement_action = QAction('Decrement Trial', self)
+        decrement_action.setShortcut('-')
+        decrement_action.setShortcutContext(Qt.WidgetShortcut)
+
+        help_url_action = QAction('Online Help', self)
+        help_url_action.triggered.connect(self.open_help_url)
+
+        about_box_action = QAction('About peyecoder', self)
+        about_box_action.setMenuRole(QAction.NoRole)
+        about_box_action.triggered.connect(self.show_about_box)
+
         export_action = QAction('E&xport CSV', self)
         export_action.setStatusTip('Export CSV')
         export_action.triggered.connect(self.export_csv)
@@ -393,6 +473,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.open_settings_action = QAction('&Settings', self)
         self.open_settings_action.setStatusTip('Open Settings Window')
+        self.open_settings_action.setMenuRole(QAction.NoRole)
         self.open_settings_action.triggered.connect(self.open_settings_dialog)
 
         # Create menu bar and add action
@@ -410,12 +491,42 @@ class MainWindow(QtWidgets.QMainWindow):
         file_menu.addAction(exit_action)
 
         edit_menu = menu_bar.addMenu('&Edit')
-        edit_menu.addAction(self.synchronize_action)
-        edit_menu.addSeparator()
         edit_menu.addAction(self.open_subject_action)
         edit_menu.addAction(self.open_occluders_action)
         edit_menu.addAction(self.open_settings_action)
 
+        controls_menu = menu_bar.addMenu('&Controls')
+        controls_menu.addAction(prescreen_action)
+        controls_menu.addAction(code_action)
+        controls_menu.addSeparator()
+        controls_menu.addAction(self.synchronize_action)
+        controls_menu.addSeparator()
+        controls_menu.addAction(next_step_action)
+        controls_menu.addAction(prev_step_action)
+        controls_menu.addSeparator()
+        controls_menu.addAction(increment_action)
+        controls_menu.addAction(decrement_action)
+
+        help_menu = menu_bar.addMenu('&Help')
+        help_menu.addAction(help_url_action)
+        help_menu.addAction(about_box_action)
+
+    def open_help_url(self):
+        target = 'https://rholson1.github.io/peyecoder/'
+        url = QtCore.QUrl(target)
+        if not QtGui.QDesktopServices.openUrl(url):
+            QMessageBox.warning(self, 'Open URL', 'Unable to open {}'.format(target))
+
+    def show_about_box(self):
+        QMessageBox.about(self, 'About peyecoder',
+                          ('<center>peyecoder {}</center>'
+                           '<p>This work was supported in part by a core grant to the Waisman Center from the '
+                           'National Institute of Child Health and Human Development (U54 HD090256), and in '
+                           'part by a grant to Susan Ellis Weismer from the National Institute on Deafness '
+                           'and Other Communication Disorders (R01 DC017974).</p>'
+                           '<p>To cite, see <a href="https://doi.org/10.5281/zenodo.3939233">https://doi.org/10.5281/zenodo.3939233</a></p>'
+                           ).format(version))
+
     def resynchronize(self):
         # Prompt user to enter a new timestamp for the current frame
         new_frame_number = self.get_timecode_frames()
@@ -475,8 +586,9 @@ class MainWindow(QtWidgets.QMainWindow):
             self.code_tab.response_box.setCurrentText(self.subject.settings['Response Keys'][e.key()])
 
         # Clear the log table when pressing a key that results in changing the position in the video
-        if e.key() in (Qt.Key_Right, Qt.Key_Left, Qt.Key_BracketLeft, Qt.Key_BracketRight, Qt.Key_Space):
-            self.logtable.clearSelection()
+        # if e.key() in (Qt.Key_Right, Qt.Key_Left, Qt.Key_BracketLeft, Qt.Key_BracketRight, Qt.Key_Space):
+        #    self.logtable.clearSelection()
+        #    self.logtable.setCurrentIndex(QtCore.QModelIndex())  # clear current index within logtable
 
     def change_selected_trials(self, delta):
         rows = self.logtable.selected_rows()
@@ -600,6 +712,9 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.show_frame()
 
+        # store frame rate for use when exporting data
+        self.subject.set_framerate(self.vid.frame_rate)
+
         # prompt for starting timecode
         self.subject.timecode_offsets[0] = self.get_timecode_frames()
         self.update_timecode()
@@ -620,6 +735,7 @@ class MainWindow(QtWidgets.QMainWindow):
             self.video_source = filename
 
             self.initialize_video()
+            self.subject.events.remove_offset(self.subject.timecode_offsets.get_offset(0))
             try:
                 self.audio.set_video_source(self.video_source, self.vid.frame_rate)
             except FileNotFoundError as e:
@@ -630,7 +746,10 @@ class MainWindow(QtWidgets.QMainWindow):
     def open_datafile(self):
         filename, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Open Data File", filter="Data Files (*.vcx)") #, QtCore.QDir.homePath())
         if filename != '':
+            self.reset_state()
             self.subject.from_plist(load_datafile(filename))
+            if self.vid:
+                self.subject.events.remove_offset(self.subject.timecode_offsets.get_offset(0))
 
             self.update_log()
             self.update_info_panel()
@@ -638,17 +757,21 @@ class MainWindow(QtWidgets.QMainWindow):
                 self.subject_dialog.update_from_dict(self.subject.to_dict())
             self.filename = filename
             self.setWindowTitle('peyecoder - {}'.format(os.path.basename(filename)))
+            self.code_tab.set_responses(list(self.subject.settings['Response Keys'].values()))
 
     def save_datafile(self):
         if self.filename:
-            save_datafile(self.filename, self.subject.to_plist())
+            if save_datafile(self.filename, self.subject.to_plist()):
+                self.subject.dirty = False
         else:
             self.save_as_datafile()
+        return not self.subject.dirty
 
     def save_as_datafile(self):
         filename = get_save_filename(self, "Save Data File", filter="Data Files (*.vcx)", default_suffix='vcx')
         if filename != '':
-            save_datafile(filename, self.subject.to_plist())
+            if save_datafile(filename, self.subject.to_plist()):
+                self.subject.dirty = False
             self.filename = filename
             self.setWindowTitle('peyecoder - {}'.format(os.path.basename(filename)))
 
@@ -743,7 +866,12 @@ class MainWindow(QtWidgets.QMainWindow):
 
     def setPosition(self, position):
         self.vid.goto_framenumber(position)
-        self.audio.seek(position)
+        try:
+            self.audio.seek(position)
+        except:
+            QMessageBox.warning(self, 'peyecoder', ('Unable to seek to the requested position in the audio.'
+                                                    ' This likely means that an incorrect timecode was entered '
+                                                    'when the video was opened.'), QMessageBox.Ok)
         self.update_timecode()
         self.show_frame()
 
