@@ -20,9 +20,9 @@ class Subject:
                   'Primary PS Complete', 'Secondary PS Complete',
                   'Sex', 'Unused Trials', 'Notes')
 
-    def __init__(self):
+    def __init__(self, parent=None):
         self._d = {}
-
+        self._framerate = 30  # default value used when no video loaded and no framerate in .vcx file
         self.occluders = Occluders()
         self.timecode_offsets = Offsets()
         self.reasons = Reasons()
@@ -39,6 +39,14 @@ class Subject:
                 int(Qt.Key_5): 'center'
             }
         }
+        self.dirty = False  # track existence of unsaved changes
+        self.parent = parent
+
+    def reset(self):
+        """Reset anything that shouldn't persist when a new data file is loaded."""
+        self.reasons = Reasons()
+        self.events = Events()
+        self.trial_order = TrialOrder()
 
     def update_from_dict(self, d):
         for f in self.fieldnames:
@@ -59,9 +67,17 @@ class Subject:
     def __getitem__(self, item):
         if item == 'Order':
             return self.trial_order.name()
+        if item == 'Framerate':
+            # return framerate of loaded video, if present.  Otherwise, return stored framerate
+            if self.parent and self.parent.vid:
+                return self.parent.vid.frame_rate
+            return self._framerate
 
         return self._d.__getitem__(item)
 
+    def set_framerate(self, framerate):
+        self._framerate = framerate
+
     def to_plist(self):
         data = {}
         data['Occluders'] = self.occluders.to_dictlist()
@@ -71,6 +87,7 @@ class Subject:
         data['Trial Order'] = self.trial_order.to_plist()
         data['Settings'] = self.settings.copy()
         data['Settings']['Response Keys'] = stringify_keys(self.settings['Response Keys'])
+        data['Framerate'] = self['Framerate']
         data.update(self.to_dict())
 
         return {'Subject': data}
@@ -91,6 +108,8 @@ class Subject:
             if 'Response Keys' in d['Settings']:
                 d['Settings']['Response Keys'] = intify_keys(d['Settings']['Response Keys'])
             self.settings.update(d['Settings'])
+        if 'Framerate' in d:
+            self._framerate = d['Framerate']
         self.update_from_dict(d)
 
 
@@ -197,6 +216,10 @@ class Reasons:
         """List of trials prescreened out by prescreener 1"""
         return [v.trial for v in self.ps[0].values() if not v._include]
 
+    def unused_reasons(self):
+        """Dictionary linking trials to reasons for exclusion"""
+        return {v.trial: v.reason for v in self.ps[0].values() if not v._include}
+
     def get_unused_display(self):
         return ', '.join([str(s) for s in self.unused()])
 
@@ -224,11 +247,23 @@ class Reasons:
 
 @total_ordering
 class Event:
-    def __init__(self, trial=0, status='', response='', frame=0):
+    def __init__(self, trial=0, status=False, response='', frame=0, has_offset=False):
+        """
+        Event object
+        :param trial: Trial number
+        :param status: Trial status (on or off)
+        :param response: Coder's response (e.g., left, right, center, off, ...)
+        :param frame: Frame number in the video
+        :param has_offset: Does the frame number have an offset for a timecode that does not start at 00:00:00:00?
+
+        has_offset will only be True for events imported from old iCoder files that specify events using a timecode
+        instead of a frame number.
+        """
         self.trial = trial
         self._status = status
         self.response = response
         self.frame = frame
+        self.has_offset = has_offset
 
     @property
     def status(self):
@@ -254,6 +289,14 @@ class Event:
         return 'Trial: {}, Status: {}, Response: {}, Frame: {}'.format(
             self.trial, self.status, self.response, self.frame)
 
+    def inverted_response(self):
+        if 'right' in self.response:
+            return self.response.replace('right', 'left')
+        elif 'left' in self.response:
+            return self.response.replace('left', 'right')
+        else:
+            return self.response
+
 
 # This version of Events only allows one event per timecode
 class EventsA:
@@ -333,16 +376,27 @@ class Events:
                     e['Timecode']['Frame'])
                 )
                 e['Frame'] = timecode.frames - 1
+                has_offset = True  # by assumption; we don't know if the timecode of the first frame is 00:00:00:00
+            else:
+                has_offset = False
 
             events.append(
                 Event(trial=e['Trial'],
                       status=e['Trial Status'] in ('on', True),
                       response=e['Type'],
-                      frame=e['Frame'])
+                      frame=e['Frame'],
+                      has_offset=has_offset)
             )
         return Events(events)
 
-    def error_items(self, unused_trials):
+    def remove_offset(self, offset):
+        """Remove offset from any events having has_offset == True"""
+        for event in self.events:
+            if event.has_offset:
+                event.frame -= offset
+                event.has_offset = False
+
+    def error_items(self, unused_trials, max_trial):
         """ Check for errors and return a list of row numbers (which should be highlighted) and
         corresponding error messages"""
         all_error_rows = []
@@ -398,6 +452,21 @@ class Events:
             all_error_rows += error_rows
             msg.append('The last event in a trial should have status "off"')
 
+        # 6. must not have 2 consecutive events with trial status 'off'
+        error_rows = []
+        for i in range(1, len(self.events)):
+            if self.events[i-1].status == self.events[i].status == 'off':
+                error_rows.append(i)
+        if error_rows:
+            all_error_rows += error_rows
+            msg.append('Cannot have consecutive events with status "off"')
+
+        # 7. Trial number should not exceed maximum trial number in trial order
+        error_rows = [i for i, e in enumerate(self.events) if e.trial > max_trial]
+        if error_rows:
+            all_error_rows += error_rows
+            msg.append('The maximum trial number in the trial order is {}'.format(max_trial))
+
         return all_error_rows, msg
 
     def __getitem__(self, item):
@@ -487,12 +556,23 @@ class Trial(dict):
             return 'No target to invert'
 
 
+def to_int(s, default=0):
+    """Try to convert an input argument to a string, returning a default value if unsuccessful."""
+    try:
+        val = int(s)
+    except ValueError:
+        val = default
+    return val
+
+
 class TrialOrder:
     def __init__(self, data=None):
         self.unused = []
+        self.max_trial = 0
         if data:
             self.data = data
             self.calc_unused()
+            self.calc_max_trial()
         else:
             self.data = []
 
@@ -508,33 +588,41 @@ class TrialOrder:
     def calc_unused(self):
         self.unused = [d['Trial Number'] for d in self.data if d['Used'] == 'no']
 
+    def calc_max_trial(self):
+        """Determine the highest-numbered trial in the trial order"""
+        self.max_trial = max([d['Trial Number'] for d in self.data])
+
     def read_trial_order(self, filename):
         """ Read data from a trial order file"""
         data = []
 
-        with open(filename, 'r', newline='') as f:
+        with open(filename, 'r', newline='', encoding='latin-1') as f:
             dialect = csv.Sniffer().sniff(f.readline(), delimiters=',\t')
             f.seek(0)
             reader = csv.DictReader(f, dialect=dialect)
 
+            # lower case of fieldnames to make case-insensitive
+            reader.fieldnames = [fieldname.lower() for fieldname in reader.fieldnames]
+
             for row in reader:
-                trial_number = int(row.get('Trial Number', 0) or row.get('trial number', 0))
+                trial_number = int(row.get('trial number', 0))
                 data.append(Trial({
-                    'Name': row.get('Name', ''),
+                    'Name': row.get('name', ''),
                     'Trial Number': trial_number,
-                    'Sound Stimulus': row.get('Sound Stimulus', ''),
-                    'Left Image': row.get('Left Image', ''),
-                    'Center Image': row.get('Center Image', ''),
-                    'Right Image': row.get('Right Image', ''),
-                    'Target Side': row.get('Target Side', '') or row.get('target side', ''),
-                    'Condition': row.get('Condition', '') or row.get('condition', ''),
-                    'Used': row.get('Used', ''),
-                    'Trial End': int(row.get('Trial End', 0) or row.get('TrEnd', 0)),
-                    'Critical Onset': int(row.get('Critical Onset', 0) or row.get('CritOnset', 0))
+                    'Sound Stimulus': row.get('sound stimulus', ''),
+                    'Left Image': row.get('left image', ''),
+                    'Center Image': row.get('center image', ''),
+                    'Right Image': row.get('right image', ''),
+                    'Target Side': row.get('target side', ''),
+                    'Condition': row.get('condition', ''),
+                    'Used': row.get('used', ''),
+                    'Trial End': to_int(row.get('trial end', 0) or row.get('trend', 0)),
+                    'Critical Onset': to_int(row.get('critical onset', 0) or row.get('critonset', 0))
                 }))
 
         self.data = data
         self.calc_unused()
+        self.calc_max_trial()
 
     def to_plist(self):
         return self.data  # maybe?
